declare type normalizedError = [key: string, value: string];
declare type onMatchCallback = (error: normalizedError, testResult: any) => void;
declare type onUnexpectedError = (error: normalizedError) => void;
declare type testCallback = (error: normalizedError) => any;
declare type expectedError = {
    test: testCallback;
    onMatch: onMatchCallback;
};
declare type changesetErrorValue = string[] | changesetError | changesetError[];
declare type changesetError = {
    [key: string]: changesetErrorValue;
};
/**
 * Process Ecto changeset errors (as a JS/TS object).
 *
 * Requires the errors object, a list of known errors (objects with a `test` predicate and an `onMatch` callback) and a handler for unexpected errors.
 *
 * Example:
  ```
  processErrors(
    {
      user: ["not authenticated"],
      type: ["must be one of public, private"],
    },
    [
      {
        test: ([k, v]) => k === "user" && v === "not authenticated",
        onMatch: ([_k, _v]) => addError("user not authenticated"),
      },
      {
        test: ([k, v]) => {
          let matches = v.match(/must be one of (.*)/g);
          if (k === "type" && matches.length > 0) return matches;
          else return false;
        },
        onMatch([_k, _v], matches) {
          let allowedTypes = matches[1].split(", ");
          addError(`types may only be ${allowedTypes}`);
        },
      },
    ],
    console.log
  );
  ```
 *
 * Complex example:
  ```
  processErrors(
    {
      name: ["invalid!"],
      nested: { user_id: ["not an int"] },
      nestedList: [{ something: ["is wrong"] }, { very: [{ nested: ["list"] }] }],
    },
    [],
    myErrorArray.push
  );

  // normalized errors
  [
    ["name", "invalid!"],
    ["nested.user_id", "not an int"],
    ["nestedList.0.something", "is wrong"],
    ["nestedList.1.very.0.nested", "list"],
  ];
  ```
 */
declare function processErrors(errorObject: changesetError, expectedErrors: expectedError[], onUnexpectedError: onUnexpectedError): normalizedError[];
/**
 * Match an error when key and value are equal. Convenience helper for `processErrors`.
 *
  ```
  processErrors(obj, [matchKeyValueEqual("user", "not found", console.log)], console.error)
  ```
 */
declare function matchKeyValueEqual(key: string, value: string, onMatch: onMatchCallback): {
    test: ([k, v]: normalizedError) => boolean;
    onMatch: onMatchCallback;
};
export { processErrors, matchKeyValueEqual };
