"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.matchKeyValueEqual = exports.processErrors = void 0;
/**
 * Process Ecto changeset errors (as a JS/TS object).
 *
 * Requires the errors object, a list of known errors (objects with a `test` predicate and an `onMatch` callback) and a handler for unexpected errors.
 *
 * Example:
  ```
  processErrors(
    {
      user: ["not authenticated"],
      type: ["must be one of public, private"],
    },
    [
      {
        test: ([k, v]) => k === "user" && v === "not authenticated",
        onMatch: ([_k, _v]) => addError("user not authenticated"),
      },
      {
        test: ([k, v]) => {
          let matches = v.match(/must be one of (.*)/g);
          if (k === "type" && matches.length > 0) return matches;
          else return false;
        },
        onMatch([_k, _v], matches) {
          let allowedTypes = matches[1].split(", ");
          addError(`types may only be ${allowedTypes}`);
        },
      },
    ],
    console.log
  );
  ```
 *
 * Complex example:
  ```
  processErrors(
    {
      name: ["invalid!"],
      nested: { user_id: ["not an int"] },
      nestedList: [{ something: ["is wrong"] }, { very: [{ nested: ["list"] }] }],
    },
    [],
    myErrorArray.push
  );

  // normalized errors
  [
    ["name", "invalid!"],
    ["nested.user_id", "not an int"],
    ["nestedList.0.something", "is wrong"],
    ["nestedList.1.very.0.nested", "list"],
  ];
  ```
 */
function processErrors(errorObject, expectedErrors, onUnexpectedError) {
    const isUnexpectedError = createUnexpectedErrorMatcher(expectedErrors);
    let normalizedErrors = parseErrorObj(errorObject || {});
    normalizedErrors.filter(isUnexpectedError).forEach(onUnexpectedError);
    return normalizedErrors;
}
exports.processErrors = processErrors;
/**
 * Match an error when key and value are equal. Convenience helper for `processErrors`.
 *
  ```
  processErrors(obj, [matchKeyValueEqual("user", "not found", console.log)], console.error)
  ```
 */
function matchKeyValueEqual(key, value, onMatch) {
    let test = ([k, v]) => k === key && v === value;
    return { test, onMatch };
}
exports.matchKeyValueEqual = matchKeyValueEqual;
/////////////
// Private //
/////////////
function createUnexpectedErrorMatcher(expectedErrors) {
    return (error) => !matchesExpectedError(error, expectedErrors);
}
function matchesExpectedError(error, expectedErrors) {
    return expectedErrors === null || expectedErrors === void 0 ? void 0 : expectedErrors.find(({ test, onMatch }) => {
        let testResult = test(error);
        if (testResult)
            onMatch(error, testResult);
        return testResult;
    });
}
// [key, obj | array-of-objects | array-of-messages]
function parseErrorObjPair([key, arrayOrObj]) {
    return Array.isArray(arrayOrObj)
        ? parseMsgOrObjArray(arrayOrObj, key)
        : parseErrorObj(arrayOrObj, key);
}
// {key, [error]}
function parseErrorObj(changesetError, parentKey) {
    return Object.entries(changesetError).flatMap(([k, v]) => parseErrorObjPair([maybePrefix(parentKey, k), v]));
}
// ["message"] OR [{nestedError: [message-or-even-deeper-nested-error]}]
function parseMsgOrObjArray(errorArray, parentKey) {
    return errorArray.flatMap((stringOrErrorObj, i) => typeof stringOrErrorObj === "string"
        ? [[parentKey, stringOrErrorObj]]
        : parseErrorObjPair([maybePrefix(parentKey, i), stringOrErrorObj]));
}
function maybePrefix(prefix, key) {
    return prefix ? `${prefix}.${key}` : `${key}`;
}
